@@ -59,6 +59,7 @@ setMethod("getTheta", "parCopula", function(copula) numeric())
 ## assign free parameter *value*s:
 ## __NOT_exported__ <= "inconsistent" with 'fixedParam<-'
 setGeneric("freeParam<-", function(copula, value) standardGeneric("freeParam<-"))
+##                                          vvvvvvvvvv
 ## set or modify "fixedness" of parameters (TRUE/FALSE) __exported__
 setGeneric("fixedParam<-", function(copula, value) standardGeneric("fixedParam<-"))
 ## logical vector indicating which parameters are free
@@ -70,5 +71,29 @@ setGeneric("nParam", function(copula, freeOnly = FALSE) standardGeneric("nParam"
 
 ##' default methods: empty parameter
 setMethod("isFree",   "parCopula", function(copula) logical())
-setMethod("nParam",   "parCopula", function(copula) 0L)
+setMethod("nParam",   "parCopula", function(copula, freeOnly) 0L)
 
+##' Get Initial Values / 'start' for the parameter(s) of a copula,
+##' similarly as \pkg{stats}' \code{\link{getInitial}()}, also related to (non-exported)
+##' \code{fitCopStart()}
+##' @param copula
+##' @param data
+##' @return numeric vector of valid (admissible) free parameters, i.e., of
+##' length and with \code{\link{names}} of \code{length(getTheta(copula, named=named))}.
+setGeneric("getIniParam", function(copula, data, default=NULL, named=TRUE, ...)
+    standardGeneric("getIniParam"), ##			^^^^^ (so can call methods w/o 'default'; bug in methods?)
+    signature = "copula")
+
+## "almost-default"       vvvvvvvvv  method :
+setMethod("getIniParam", "parCopula", function(copula, data, default=NULL, named=TRUE, ...) {
+    clc <- class(copula)
+    if(is.null(default)) default <- rep(NA_real_, nParam(copula, freeOnly=TRUE))
+    if(hasMethod("iTau", clc)) {
+        getIni_itau(copula, data, default=default, classDef=getClass(clc), ...)
+        ##========= --> ./fitCopula.R
+    } else {
+        warning(gettextf("no method to get initial parameter for copula of class \"%s\"", clc),
+                domain=NA)
+        default
+    }
+})
