@@ -62,7 +62,7 @@ schlatherCopula <- function(param = NA_real_) {
              dimension = dim,
              ## exprdist = c(cdf = cdf, pdf = pdf),
              parameters = param[1],
-             param.names = "param",
+             param.names = "alpha",
              param.lowbnd = -1,
              param.upbnd = 1,
              fullname = "<deprecated slot>")# = "Schlather copula family; Extreme value copula"
